import copy

from google.protobuf.struct_pb2 import Struct
from perxis.items import items_pb2


ACTIONS_COLLECTION_ID = "space_actions"


def process_action_id(action_id: str) -> str:
    action_id = action_id.lower().replace("-", "_").replace(" ", "_")

    return action_id


def make_action_dict(
        extension_id: str,
        action_id: str,
        name: str,
        kind: int,
        **kwargs,
) -> dict:
    action_id = process_action_id(action_id)

    return {
        "id": action_id,
        "action": f"grpc:///{extension_id}/{action_id}",
        "name": name,
        "kind": kind,
        **kwargs,
    }


def make_action_struct(data: dict) -> Struct:
    struct = Struct()
    struct.update(data)

    return struct


def make_action_item(space_id: str, env_id: str, data: dict) -> items_pb2.Item:
    copied_data = copy.deepcopy(data)
    action_id = copied_data.pop("id")

    struct = make_action_struct(data)

    return items_pb2.Item(
        id=action_id,
        space_id=space_id,
        env_id=env_id,
        collection_id=ACTIONS_COLLECTION_ID,
        data=struct
    )
